--[[--
@module MyDungeonsBook
]]

--[[--
UI
@section UI
]]
local AceGUI = LibStub("AceGUI-3.0");
--[[--
Creates a frame for DEV tab.

@param[type=Frame] parentFrame
@param[type=number] challengeId
@return[type=Frame] devFrame
]]
function MyDungeonsBook:DevFrame_Create(parentFrame, challengeId)
	local devFrame = self:TabContentWrapperWidget_Create(parentFrame);
	devFrame:SetFullHeight(true);
	local challenge = self.db.char.challenges[challengeId];
	if (challenge) then
	    local mechanics = challenge.mechanics;
	    local mechanicsDropdown = AceGUI:Create("Dropdown");
	    if (type(mechanics) == "table") then
            for k, _ in pairs(mechanics) do
                mechanicsDropdown:AddItem(k, k);
            end
        end
        devFrame:AddChild(mechanicsDropdown);
		local editBox = AceGUI:Create("EditBox");
		editBox:SetFullWidth(true);
		editBox.button:Hide();
		devFrame:AddChild(editBox);
		mechanicsDropdown:SetCallback("OnValueChanged", function(_, _, key)
            local isTable = type(challenge.mechanics[key]) == "table";
            local text = self:Table2Json((isTable and challenge.mechanics[key]) or select(2, self:Decompress(challenge.mechanics[key])));
            editBox:SetText(text);
        end);
		local mechanicsSizeEditBox = AceGUI:Create("MultiLineEditBox");
		mechanicsSizeEditBox:SetNumLines(10);
		local mechanicsSize = "";
		if (type(mechanics) == "table") then
		    for k, v in pairs(mechanics) do
		        local isTable = type(v) == "table";
		        mechanicsSize = string.format("%s%s: %s\n", mechanicsSize, k, string.len(isTable and self:Compress(v) or v));
		    end
		end
		mechanicsSizeEditBox:SetFullWidth(true);
		mechanicsSizeEditBox:SetLabel("Mechanics");
		mechanicsSizeEditBox:SetText(mechanicsSize);
		devFrame:AddChild(mechanicsSizeEditBox);
	end
	return devFrame;
end
