--[[--
@module MyDungeonsBook
]]

--[[--
UI
@section UI
]]

local L = LibStub("AceLocale-3.0"):GetLocale("MyDungeonsBook");
local AceGUI = LibStub("AceGUI-3.0");

--[[--
Creates a frame for Loot tab

@param[type=Frame] parentFrame
@param[type=number] challengeId
@return[type=Frame]
]]
function MyDungeonsBook:LootFrame_Create(parentFrame, challengeId)
    local lootFrame = self:TabContentWrapperWidget_Create(parentFrame);
    local challenge = self:Challenge_GetById(challengeId);
    if (not challenge) then
        return lootFrame;
    end
    local loot = challenge.misc.loot;
    if (not loot) then
        return lootFrame;
    end
    for _, unitId in pairs(self:GetPartyRoster()) do
        local name, nameAndRealm = self:GetNameByPartyUnit(challengeId, unitId);
        local personalLoot = loot[name] or loot[nameAndRealm];
        if (personalLoot) then
            for indx, lootItem in pairs(personalLoot) do
                self:LootFrame_LootForPartyMemberFrame_Create(lootFrame, challengeId, unitId, lootItem, indx == 1);
            end
        end
    end
    return lootFrame;
end

--[[--
@param[type=Frame] parentFrame
@param[type=number] challengeId
@param[type=unitId] unitId
@param[type=table] lootItem
@param[type=boolean] showName
@return[type=Frame]
]]
function MyDungeonsBook:LootFrame_LootForPartyMemberFrame_Create(parentFrame, challengeId, unitId, lootItem, showName)
    local challenge = self:Challenge_GetById(challengeId);
    -- Party member name
    local nameFrame = AceGUI:Create("Label");
    local _, itemId = strsplit(":", lootItem[2]);
    if (not itemId) then
        return;
    end
    parentFrame:AddChild(nameFrame);
    if (challenge.players[unitId] and showName) then
        nameFrame:SetText(self:GetUnitNameRealmRoleStr(challenge.players[unitId]) or L["Not Found"]);
    end
    nameFrame:SetWidth(200);
    -- Amount frame
    local amountLabel = AceGUI:Create("Label");
    local amount = lootItem[3];
    amountLabel:SetWidth(40);
    amountLabel:SetHeight(30);
    parentFrame:AddChild(amountLabel);
    if (type(amount) == "number" and amount > 1) then
        amountLabel:SetText(amount .. " x");
    end

    local function lootItemHover(frame)
        GameTooltip:SetOwner(frame, "ANCHOR_NONE");
        GameTooltip:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT");
        GameTooltip:SetHyperlink(lootItem[2]);
        GameTooltip:Show();
    end

    -- Item icon Frame
    local lootFrame = AceGUI:Create("InteractiveLabel");
    lootFrame:SetText("|T" .. GetItemIcon(itemId) .. self:GetIconTextureSuffix(30) .. "|t");
    lootFrame:SetWidth(60);
    lootFrame:SetHeight(30);
    parentFrame:AddChild(lootFrame);
    lootFrame:SetCallback("OnEnter", function(f)
        lootItemHover(f.frame);
    end);
    lootFrame:SetCallback("OnLeave", function()
        self:Table_Cell_MouseOut();
    end);
    -- Item name frame
    local itemNameFrame = AceGUI:Create("InteractiveLabel");
    itemNameFrame:SetWidth(200);
    itemNameFrame:SetHeight(30);
    itemNameFrame:SetText(lootItem[2]);
    parentFrame:AddChild(itemNameFrame);
    itemNameFrame:SetCallback("OnEnter", function(f)
        lootItemHover(f.frame);
    end);
    itemNameFrame:SetCallback("OnLeave", function()
        self:Table_Cell_MouseOut();
    end);
    self:NewLine_Create(parentFrame);
    return nameFrame;
end
