--[[--
@module MyDungeonsBook
]]

--[[--
UI
@section UI
]]

local L = LibStub("AceLocale-3.0"):GetLocale("MyDungeonsBook");

local function getOwnSpellMenu(rows, index, cols, challengeId)
    local challenge = MyDungeonsBook:Challenge_GetById(challengeId);
    local instanceId = challenge.challengeInfo.currentZoneId;
    local affixes = challenge.challengeInfo.affixes;
    local bosses = MyDungeonsBook:GetDungeonMetaInfo(instanceId).encounters;
    local spellId = rows[index].cols[1].value;
    local spellName = GetSpellInfo(spellId);
    local npcId = rows[index].cols[4].value;
    local npcName = MyDungeonsBook:GetNpcNameByNpcId(npcId);
    local npcMenu = MyDungeonsBook:WowHead_Menu_Npc(npcId);
    local spellMenu = MyDungeonsBook:WowHead_Menu_Spell(spellId);
    return {
        {
            text = L["WowHead"],
            hasArrow = true,
            menuList = {
                {
                    text = spellName,
                    func = spellMenu.func
                },
                {
                    text = npcName,
                    func = npcMenu.func
                }
            }
        },
        {
            text = L["Weak Auras"],
            hasArrow = true,
            menuList = {
                MyDungeonsBook:Wa_ContextMenuItem_Frontal_Create(spellId, instanceId, bosses, affixes),
                MyDungeonsBook:Wa_ContextMenuItem_Dodge_Create(spellId, instanceId, bosses, affixes),
                MyDungeonsBook:Wa_ContextMenuItem_Interrupt_Create(spellId, instanceId, bosses, affixes),
                MyDungeonsBook:Wa_ContextMenuItem_CastOnMe_Create(spellId, instanceId, bosses, affixes),
                MyDungeonsBook:Wa_ContextMenuItem_CustomActionOnSpellStart_Create(spellId, instanceId, bosses, affixes),
                MyDungeonsBook:Wa_ContextMenuItem_CustomActionOnSpellSuccess_Create(spellId, instanceId, bosses, affixes)
            }
        }
    };
end

--[[--
Create a frame for Special Casts tab (data is taken from `mechanics[**-CASTS-DONE-BY-PARTY-MEMBERS]`).

Mouse hover/out handler are included.

@param[type=Frame] parentFrame
@param[type=number] challengeId
]]
function MyDungeonsBook:OwnCastsByEnemiesFrame_Create(parentFrame, challengeId)
    local ownCastsByEnemiesFrame = self:TabContentWrapperWidget_Create(parentFrame);
    local data = self:OwnCastsByEnemiesFrame_GetDataForTable(challengeId);
    local columns = self:OwnCastsByEnemiesFrame_GetColumnsForTable(challengeId);
    local table = self:TableWidget_Create(columns, 12, 40, nil, ownCastsByEnemiesFrame, "own-casts-by-enemies");
    table:SetData(data);
    table:RegisterEvents({
        OnClick = function(_, _, data, _, _, realrow, column, _, button)
            if (button == "RightButton" and realrow) then
                EasyMenu(getOwnSpellMenu(data, realrow, table.cols, challengeId), self.menuFrame, "cursor", 0 , 0, "MENU");
            end
            self:Table_Cell_ShiftClick_Check(data, realrow, column, button, 3);
        end,
        OnEnter = function (_, cellFrame, data, _, _, realrow, column)
            if (realrow) then
                if (column == 2 or column == 3) then
                    self:Table_Cell_SpellMouseHover(cellFrame, data[realrow].cols[1].value);
                end
            end
        end,
        OnLeave = function (_, _, _, _, _, realrow, column)
            if (realrow) then
                if (column == 2 or column == 3) then
                    self:Table_Cell_MouseOut();
                end
            end
        end
    });
    return ownCastsByEnemiesFrame;
end

function MyDungeonsBook:OwnCastsByEnemiesFrame_GetColumnsForTable(challengeId)
    return {
        {
            name = " ",
            width = 1,
            align = "LEFT"
        },
        {
            name = "",
            width = 40,
            align = "LEFT",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsSpellIcon(...);
            end
        },
        {
            name = L["Spell"],
            width = 150,
            align = "LEFT",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsSpellLink(...);
            end
        },
        {
            name = "",
            width = 1,
            align = "LEFT"
        },
        {
            name = "",
            width = 40,
            align = "LEFT",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsTexture(...);
            end
        },
        {
            name = L["NPC"],
            width = 200,
            align = "LEFT",
            bgcolor = {
                r = 0,
                g = 0,
                b = 0,
                a = 0.4
            },
        },
        {
            name = L["Count"],
            width = 40,
            align = "RIGHT"
        },
        {
            name = L["Kickable"],
            width = 40,
            align = "CENTER",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsEjIcon("interrupt", ...);
            end
        }
    };
end

function MyDungeonsBook:OwnCastsByEnemiesFrame_GetDataForTable(challengeId)
    local challenge = self:Challenge_GetById(challengeId);
    local enemiesCasts = self:Challenge_Mechanic_GetById(challengeId, "ALL-ENEMY-PASSED-CASTS");
    if (not enemiesCasts) then
        return {};
    end
    local spellsData = {};
    local tableData = {};
    local interruptsMechanics = self:Challenge_Mechanic_GetById(challengeId, "COMMON-INTERRUPTS");
    for _, interruptsBySpells in pairs(interruptsMechanics) do
        for _, kickedSpells in pairs(interruptsBySpells) do
            for spellId, kickedCount in pairs(kickedSpells) do
                spellsData[spellId] = spellsData[spellId] or 0;
                spellsData[spellId] = spellsData[spellId] + kickedCount;
            end
        end
    end
    for spellId, castedCount in pairs(enemiesCasts) do
        spellsData[spellId] = spellsData[spellId] or 0;
        spellsData[spellId] = spellsData[spellId] + castedCount;
    end
    for spellId, castedCount in pairs(spellsData) do
        local npcs = (self.db.global.meta.spells[spellId] and self.db.global.meta.spells[spellId].casters) or nil;
        local mdtEnemiesDb =  IsAddOnLoaded("MythicDungeonTools") and self:Mdt_GetInstanceEnemiesRemapped(challenge.challengeInfo.currentZoneId) or {};
        local npcId = -1;
        if (npcs) then
            for nId, _ in pairs(npcs) do
                npcId = nId;
                break;
            end
        end
        local npcName = self:GetNpcNameByNpcId(npcId);
        local mdtOverrides = self.db.global.meta.addons.mythicDungeonTools;
        local enemyInfo = mdtOverrides.npcs[npcId] or mdtEnemiesDb[npcId] or {};
        local enemyPortraitDisplayId = enemyInfo.displayId or -1;
        tinsert(tableData, {
            cols = {
                {value = spellId},
                {value = spellId},
                {value = spellId},
                {value = npcId},
                {value = enemyPortraitDisplayId},
                {value = npcName},
                {value = castedCount},
                {value = self.db.global.meta.spells[spellId] and self.db.global.meta.spells[spellId].kickable and 1 or 0}
            }
        });
    end
    return tableData;
end
