--[[--
@module MyDungeonsBook
]]

--[[--
UI
@section UI
]]

local L = LibStub("AceLocale-3.0"):GetLocale("MyDungeonsBook");

--[[--
Creates tabs (with click-handlers) for Roster frame.

@param[type=Frame] parentFrame
@return[type=Frame] tabsButtonsFrame
]]
function MyDungeonsBook:RosterFrame_CreateTabButtonsFrame(parentFrame, challengeId)
    local tabs = self:TabsWidget_Create(parentFrame);
    local tabsConfig = {
        {value = "shortInfo", text = L["Info"]},
        {value = "equipment", text = L["Equipment"]},
    };
    local challenge = self:Challenge_GetById(challengeId);
    if (not challenge) then
        return tabs;
    end
    if (challenge.gameInfo and challenge.gameInfo.version) then
        local version = challenge.gameInfo.version;
        local major = string.sub(version, 1, string.find(version, "%.") - 1);
        if (tonumber(major, 10) >= 10) then
            tinsert(tabsConfig, {value = "talents", text = L["Talents"]});
        else
            tinsert(tabsConfig, {value = "oldTalents", text = L["Talents"]});
        end
    else
    end
    -- tinsert(tabsConfig, {value = "covenant", text = L["Covenant"]});
    tinsert(tabsConfig, {value = "misc", text = L["Misc"]});
    tabs:SetTabs(tabsConfig);
    tabs:SetCallback("OnGroupSelected", function (container, _, tabId)
        container:ReleaseChildren();
        if (tabId == "covenant") then
            self.covenantFrame = self:CovenantFrame_Create(container, self.activeChallengeId);
        end
        if (tabId == "shortInfo") then
            self.covenantFrame = self:ShortInfoFrame_Create(container, self.activeChallengeId);
        end
        if (tabId == "equipment") then
            self.equipmentFrame = self:EquipmentFrame_Create(container, self.activeChallengeId);
        end
        if (tabId == "oldTalents") then
            self.talentsFrame = self:TalentsFrame_Create(container, self.activeChallengeId);
        end
        if (tabId == "talents") then
            self.talentsFrame = self:TalentsPartyMembersFrame_Create(container, self.activeChallengeId);
        end
        if (tabId == "misc") then
            self.talentsFrame = self:MiscFrame_Create(container, self.activeChallengeId);
        end
    end);
    tabs:SetHeight(622);
    return tabs;
end
