--[[--
@module MyDungeonsBook
]]

--[[--
UI
@section UI
]]

local L = LibStub("AceLocale-3.0"):GetLocale("MyDungeonsBook");
local AceGUI = LibStub("AceGUI-3.0");

--[[--
@param[type=Frame] parentFrame
@param[type=number] challengeId
@param[type=unitId] unitId
@return[type=Frame]
]]
function MyDungeonsBook:TalentsPartyMemberFrame_Create(parentFrame, challengeId, unitId)
    local talentsPartyMemberFrame = self:TabContentWrapperWidget_Create(parentFrame);
    local unitInfo = self:Challenge_GetById(challengeId).players[unitId];
    local talents = unitInfo and unitInfo.talents or {};
    self:TalentsPartyMemberFrame_TalentsFrame_Create(talentsPartyMemberFrame, talents, "class", GetClassInfo(unitInfo.class));
    self:TalentsPartyMemberFrame_TalentsFrame_Create(talentsPartyMemberFrame, talents, "spec", select(2, GetSpecializationInfoByID(unitInfo.spec)));
    return talentsPartyMemberFrame;
end

function MyDungeonsBook:TalentsPartyMemberFrame_TalentsFrame_Create(parentFrame, talents, group, title)
    local wrapperFrame = AceGUI:Create("InlineGroup");
    wrapperFrame:SetLayout("Flow");
    wrapperFrame:SetFullWidth(true);

    wrapperFrame:SetTitle(title);
    local talentsInfo = {};
    for _, v in pairs(talents) do tinsert(talentsInfo, v) end
    table.sort(talentsInfo, function (t1, t2)
        return tonumber(t1.spellId) < tonumber(t2.spellId);
    end);
    local index = 1;
    for _, talentInfo in pairs(talentsInfo) do
        if (talentInfo.group == group) then
            local spellId = talentInfo.spellId;
            local icon = select(8, GetSpellInfo(spellId));
            local rankFrame = AceGUI:Create("Icon");
            rankFrame:SetLabel(talentInfo.activeRank);
            rankFrame:SetImage(icon, .1,.9,.1,.9);
            rankFrame:SetWidth(45);
            rankFrame:SetImageSize(35, 35);
            rankFrame:SetCallback("OnEnter", function()
                self:Table_Cell_SpellMouseHover(rankFrame.frame, spellId);
            end);
            rankFrame:SetCallback("OnLeave", function()
                self:Table_Cell_MouseOut();
            end);
            rankFrame.label:SetPoint("TOPRIGHT", rankFrame.frame, "TOPRIGHT", -5, -5);
            rankFrame.label:SetJustifyH("RIGHT");
            rankFrame.label:SetFont(NumberFontNormalYellow:GetFont(), 15, "OUTLINE");
            rankFrame:SetHeight(45);
            wrapperFrame:AddChild(rankFrame);
            if (index % 10 == 0) then
                self:NewLine_Create(wrapperFrame);
            end
            index = index + 1;
        end
    end
    parentFrame:AddChild(wrapperFrame);
    return wrapperFrame;
end
