--[[--
@module MyDungeonsBook
]]

--[[--
Utils
@section Utils
]]

local customTrigger = [[function (event, ...)
    local _, subevent = ...;
    local targetGUID = select(11, ...);
    local spellId = select(12, ...);

    if (subevent == "SPELL_CAST_START" and UnitGUID("player") == targetGUID and spellId == SPELL_ID) then
        return true;
    end
end]];

function MyDungeonsBook:WA_CastOnMe_Template_Get()
    return {
        ["iconSource"] = -1,
        ["color"] = {
            [1] = 1,
            [2] = 1,
            [3] = 1,
            [4] = 1,
        },
        ["yOffset"] = 0,
        ["anchorPoint"] = "CENTER",
        ["cooldownSwipe"] = true,
        ["cooldownEdge"] = false,
        ["actions"] = {
            ["start"] = {
                ["sound"] = "Interface\\AddOns\\WeakAuras\\Media\\Sounds\\OhNo.ogg",
                ["do_sound"] = true,
            },
            ["init"] = {
            },
            ["finish"] = {
            },
        },
        ["triggers"] = {
            [1] = {
                ["trigger"] = {
                    ["type"] = "custom",
                    ["subeventSuffix"] = "_CAST_START",
                    ["custom"] = customTrigger,
                    ["duration"] = "",
                    ["event"] = "Combat Log",
                    ["subeventPrefix"] = "SPELL",
                    ["events"] = "UNIT_SPELLCAST_START UNIT_SPELLCAST_CHANNEL_START",
                    ["use_spellId"] = false,
                    ["spellIds"] = {
                    },
                    ["custom_hide"] = "timed",
                    ["custom_type"] = "event",
                    ["names"] = {
                    },
                    ["unit"] = "player",
                    ["debuffType"] = "HELPFUL",
                },
                ["untrigger"] = {
                },
            },
            ["activeTriggerMode"] = -10,
        },
        ["internalVersion"] = 51,
        ["keepAspectRatio"] = false,
        ["selfPoint"] = "CENTER",
        ["desaturate"] = false,
        ["subRegions"] = {
            [1] = {
                ["type"] = "subbackground",
            },
            [2] = {
                ["text_shadowXOffset"] = 0,
                ["text_text_format_s_format"] = "none",
                ["text_text"] = "",
                ["text_shadowColor"] = {
                    [1] = 0,
                    [2] = 0,
                    [3] = 0,
                    [4] = 1,
                },
                ["text_selfPoint"] = "AUTO",
                ["text_automaticWidth"] = "Auto",
                ["text_fixedWidth"] = 64,
                ["anchorYOffset"] = 0,
                ["text_justify"] = "CENTER",
                ["rotateText"] = "NONE",
                ["type"] = "subtext",
                ["text_color"] = {
                    [1] = 1,
                    [2] = 1,
                    [3] = 1,
                    [4] = 1,
                },
                ["text_font"] = "Friz Quadrata TT",
                ["text_shadowYOffset"] = 0,
                ["text_wordWrap"] = "WordWrap",
                ["text_visible"] = true,
                ["text_anchorPoint"] = "OUTER_BOTTOM",
                ["text_fontSize"] = 32,
                ["anchorXOffset"] = 0,
                ["text_fontType"] = "OUTLINE",
            },
            [3] = {
                ["glowFrequency"] = 0.25,
                ["type"] = "subglow",
                ["glowXOffset"] = 0,
                ["glowType"] = "Pixel",
                ["glowLength"] = 10,
                ["glowYOffset"] = 0,
                ["glowColor"] = {
                    [1] = 0.77647058823529,
                    [2] = 0.6078431372549,
                    [3] = 0.42745098039216,
                    [4] = 1,
                },
                ["useGlowColor"] = true,
                ["glow"] = true,
                ["glowScale"] = 1,
                ["glowThickness"] = 3,
                ["glowLines"] = 8,
                ["glowBorder"] = false,
            },
        },
        ["height"] = 128,
        ["load"] = {
            ["size"] = {
                ["multi"] = {
                },
            },
        },
        ["regionType"] = "icon",
        ["zoom"] = 0.3,
        ["cooldown"] = true,
        ["animation"] = {
            ["start"] = {
                ["type"] = "none",
                ["easeStrength"] = 3,
                ["duration_type"] = "seconds",
                ["easeType"] = "none",
            },
            ["main"] = {
                ["type"] = "none",
                ["easeStrength"] = 3,
                ["duration_type"] = "seconds",
                ["easeType"] = "none",
            },
            ["finish"] = {
                ["type"] = "none",
                ["easeStrength"] = 3,
                ["duration_type"] = "seconds",
                ["easeType"] = "none",
            },
        },
        ["cooldownTextDisabled"] = false,
        ["authorOptions"] = {
        },
        ["xOffset"] = 0,
        ["id"] = "",
        ["config"] = {
        },
        ["alpha"] = 1,
        ["width"] = 128,
        ["frameStrata"] = 1,
        ["uid"] = "",
        ["inverse"] = false,
        ["anchorFrameType"] = "SCREEN",
        ["conditions"] = {
        },
        ["information"] = {
        },
        ["icon"] = true,
    };
end

function MyDungeonsBook:WA_CastOnMe_Aura_Create(auraData, instanceId, bossId, affixId, customText)
    local wa = self:WA_CastOnMe_Template_Get();
    local spellId = auraData.spellId;
    local _, _, icon, castTime = GetSpellInfo(spellId);
    wa.triggers[1].trigger.custom = string.gsub(wa.triggers[1].trigger.custom, "SPELL_ID", string.format("%s", spellId));
    wa.triggers[1].trigger.duration = string.format("%s", castTime / 1000);
    wa.displayIcon = icon;
    wa.subRegions[2].text_text = customText;
    wa.id = self:Wa_GenerateId(auraData, instanceId, bossId, affixId, "I'm target");

    return self:Wa_AddNewWeakAura(wa, auraData, instanceId, bossId, affixId);
end

function MyDungeonsBook:Wa_ContextMenuItem_CastOnMe_Create(spellId, instanceId, bosses, affixes)
    local menuItemsForBosses = {};
    for bossId, bossInfo in pairs(bosses) do
        tinsert(menuItemsForBosses, {
            text = bossInfo.name,
            func = function()
                StaticPopup_Show("MDB_WA_CUSTOM_TEXT_INPUT", "", "", {
                    auraData = {spellId = spellId},
                    bossId = bossId,
                    instanceId = instanceId,
                    type = "CAST_ON_ME"
                });
            end
        });
    end
    local menuItemsForAffixes = {};
    for _, affixId in pairs(affixes) do
        tinsert(menuItemsForAffixes, {
            text = self:GetAffixMetaInfo(affixId).name,
            func = function()
                StaticPopup_Show("MDB_WA_CUSTOM_TEXT_INPUT", "", "", {
                    auraData = {spellId = spellId},
                    affixId = affixId,
                    type = "CAST_ON_ME"
                });
            end
        });
    end
    return {
        text = "I'm target",
        hasArrow = true,
        menuList = {
            {
                text = "Trash",
                func = function()
                    StaticPopup_Show("MDB_WA_CUSTOM_TEXT_INPUT", "", "", {
                        auraData = { spellId = spellId },
                        instanceId = instanceId,
                        type = "CAST_ON_ME"
                    });
                end
            },
            {
                text = "Boss",
                hasArrow = true,
                menuList = menuItemsForBosses
            },
            {
                text = "Affix",
                hasArrow = true,
                menuList = menuItemsForAffixes
            }
        }
    };
end
