--[[--
@module MyDungeonsBook
]]

--[[--
Utils
@section Utils
]]

function MyDungeonsBook:WA_Frontal_Template_Get()
    return {
        ["iconSource"] = -1,
        ["yOffset"] = 0,
        ["anchorPoint"] = "CENTER",
        ["cooldownSwipe"] = true,
        ["cooldownEdge"] = false,
        ["icon"] = true,
        ["triggers"] = {
            [1] = {
                ["trigger"] = {
                    ["subeventPrefix"] = "SPELL",
                    ["type"] = "combatlog",
                    ["spellId"] = "",
                    ["subeventSuffix"] = "_CAST_START",
                    ["unit"] = "player",
                    ["duration"] = "",
                    ["event"] = "Combat Log",
                    ["use_sourceFlags2"] = true,
                    ["sourceFlags2"] = "Hostile",
                    ["use_spellId"] = true,
                    ["spellIds"] = {
                    },
                    ["use_sourceUnit"] = false,
                    ["names"] = {
                    },
                    ["use_sourceFlags"] = false,
                    ["use_cloneId"] = true,
                    ["debuffType"] = "HELPFUL",
                },
                ["untrigger"] = {
                },
            },
            ["activeTriggerMode"] = -10,
        },
        ["internalVersion"] = 1,
        ["keepAspectRatio"] = false,
        ["selfPoint"] = "CENTER",
        ["desaturate"] = false,
        ["subRegions"] = {
            [1] = {
                ["type"] = "subbackground",
            },
            [2] = {
                ["text_shadowXOffset"] = 0,
                ["text_text_format_s_format"] = "none",
                ["text_text"] = "Frontal!",
                ["text_shadowColor"] = {
                    [1] = 0,
                    [2] = 0,
                    [3] = 0,
                    [4] = 1,
                },
                ["text_selfPoint"] = "AUTO",
                ["text_automaticWidth"] = "Auto",
                ["text_fixedWidth"] = 64,
                ["anchorYOffset"] = 0,
                ["text_justify"] = "CENTER",
                ["rotateText"] = "NONE",
                ["type"] = "subtext",
                ["text_color"] = {
                    [1] = 1,
                    [2] = 1,
                    [3] = 1,
                    [4] = 1,
                },
                ["text_font"] = "Friz Quadrata TT",
                ["text_shadowYOffset"] = 0,
                ["text_wordWrap"] = "WordWrap",
                ["text_visible"] = true,
                ["text_anchorPoint"] = "OUTER_BOTTOM",
                ["text_fontSize"] = 32,
                ["anchorXOffset"] = 0,
                ["text_fontType"] = "OUTLINE",
            },
            [3] = {
                ["glowFrequency"] = 0.25,
                ["type"] = "subglow",
                ["useGlowColor"] = true,
                ["glowType"] = "Pixel",
                ["glowLength"] = 10,
                ["glowYOffset"] = 0,
                ["glowColor"] = {
                    [1] = 0.76470588235294,
                    [2] = 0.11764705882353,
                    [3] = 0.22745098039216,
                    [4] = 1,
                },
                ["glowXOffset"] = 0,
                ["glow"] = true,
                ["glowThickness"] = 3,
                ["glowScale"] = 1,
                ["glowLines"] = 8,
                ["glowBorder"] = false,
            },
        },
        ["height"] = 128,
        ["load"] = {
            ["size"] = {
                ["multi"] = {
                },
            },
        },
        ["regionType"] = "icon",
        ["displayIcon"] = -1,
        ["animation"] = {
            ["start"] = {
                ["easeStrength"] = 3,
                ["type"] = "none",
                ["duration_type"] = "seconds",
                ["easeType"] = "none",
            },
            ["main"] = {
                ["easeStrength"] = 3,
                ["type"] = "none",
                ["duration_type"] = "seconds",
                ["easeType"] = "none",
            },
            ["finish"] = {
                ["easeStrength"] = 3,
                ["type"] = "none",
                ["duration_type"] = "seconds",
                ["easeType"] = "none",
            },
        },
        ["frameStrata"] = 1,
        ["information"] = {
        },
        ["authorOptions"] = {
        },
        ["cooldownTextDisabled"] = false,
        ["actions"] = {
            ["start"] = {
                ["sound"] = "Interface\\AddOns\\WeakAuras\\Media\\Sounds\\AirHorn.ogg",
                ["do_sound"] = true,
            },
            ["finish"] = {
            },
            ["init"] = {
            },
        },
        ["zoom"] = 0.3,
        ["id"] = "",
        ["xOffset"] = 0,
        ["alpha"] = 1,
        ["width"] = 128,
        ["uid"] = "",
        ["config"] = {
        },
        ["inverse"] = false,
        ["anchorFrameType"] = "SCREEN",
        ["conditions"] = {
        },
        ["cooldown"] = true,
        ["color"] = {
            [1] = 1,
            [2] = 1,
            [3] = 1,
            [4] = 1,
        },
    };
end

--[[--
@param[type=table] auraData
@param[type=number|nil] instanceId
@param[type=number|nil] bossId
@param[type=number|nil] affixId
@return[type=boolean]
]]
function MyDungeonsBook:WA_Frontal_Aura_Create(auraData, instanceId, bossId, affixId)
    local wa = self:WA_Frontal_Template_Get();
    local spellId = auraData.spellId;
    local _, _, icon, castTime = GetSpellInfo(spellId);
    wa.triggers[1].trigger.spellId = string.format("%s", spellId);
    wa.triggers[1].trigger.duration = string.format("%s", castTime / 1000);
    wa.displayIcon = icon;
    wa.id = self:Wa_GenerateId(auraData, instanceId, bossId, affixId, "Frontal");

    return self:Wa_AddNewWeakAura(wa, auraData, instanceId, bossId, affixId);
end

function MyDungeonsBook:Wa_ContextMenuItem_Frontal_Create(spellId, instanceId, bosses, affixes)
    local menuItemsForBosses = {};
    for bossId, bossInfo in pairs(bosses) do
        tinsert(menuItemsForBosses, {
            text = bossInfo.name,
            func = function()
                self:WA_Frontal_Aura_Create({spellId = spellId}, instanceId, bossId);
            end
        });
    end
    local menuItemsForAffixes = {};
    for _, affixId in pairs(affixes) do
        tinsert(menuItemsForAffixes, {
            text = self:GetAffixMetaInfo(affixId).name,
            func = function()
                self:WA_Frontal_Aura_Create({ spellId = spellId }, nil, nil, affixId);
            end
        });
    end
    return {
        text = "Frontal",
        hasArrow = true,
        menuList = {
            {
                text = "Trash",
                func = function()
                    self:WA_Frontal_Aura_Create({ spellId = spellId }, instanceId, nil, nil);
                end
            },
            {
                text = "Boss",
                hasArrow = true,
                menuList = menuItemsForBosses
            },
            {
                text = "Affix",
                hasArrow = true,
                menuList = menuItemsForAffixes
            }
        }
    };
end
